# Copyright 1999-2006 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

RC_GOT_FUNCTIONS="yes"

# Override defaults with user settings ...
[[ -f /etc/conf.d/rc ]] && . /etc/conf.d/rc

# Check /etc/conf.d/rc for a description of these ...
declare -r svclib="/lib/rcscripts"
declare -r svcdir="${svclib}/init.d"
svcsize="${svcsize:-1024}"

# Different types of dependencies
deptypes="need use"
# Different types of order deps
ordtypes="before after"

#
# Internal variables
#

# Script verbosity 
RC_VERBOSE="${RC_VERBOSE:-no}"
RC_QUIET="${RC_QUIET:-no}"
RC_QUIET_STDOUT="${RC_QUIET_STDOUT:-no}"

# Should we use color?
RC_NOCOLOR="${RC_NOCOLOR:-no}"
# Can the terminal handle endcols?
RC_ENDCOL="yes"

#
# Default values for rc system
#
RC_TTY_NUMBER="${RC_TTY_NUMBER:-11}"
RC_PARALLEL_STARTUP="${RC_PARALLEL_STARTUP:-no}"
RC_USE_FSTAB="${RC_USE_FSTAB:-no}"
RC_USE_CONFIG_PROFILE="${RC_USE_CONFIG_PROFILE:-yes}"
RC_FORCE_AUTO="${RC_FORCE_AUTO:-no}"
RC_DEVICES="${RC_DEVICES:-auto}"
RC_DOWN_INTERFACE="${RC_DOWN_INTERFACE:-yes}"
RC_DOWN_HARDDISK="${RC_DOWN_HARDDISK:-yes}"
RC_VOLUME_ORDER="${RC_VOLUME_ORDER:-raid evms lvm dm}"
RC_CRITICAL_SERVICES="${RC_CRITICAL_SERVICES:-checkroot}"

#
# Default values for e-message indentation and dots
#
RC_INDENTATION=''
RC_DEFAULT_INDENT=2
#RC_DOT_PATTERN=' .'
RC_DOT_PATTERN=''

# Setup i18n variables
TEXTDOMAINDIR="${svclib}/locale"
TEXTDOMAIN="rc-scripts"

# Ensure that _ebuffer is emtpy
_ebuffer=

# bool has_addon(addon)
#
#   See if addon exists.
#
has_addon() {
	[[ -e ${svclib}/addons/$1 ]]
}

# void import_addon(char *addon)
#
#  Import code from the specified addon if it exists
#
import_addon() {
	local addon="${svclib}/addons/$1"
	if has_addon $1 ; then
		source "${addon}"
		return 0
	fi
	return 1
}

# void splash(...)
#
#  Notify bootsplash/splashutils/gensplash/whatever about
#  important events.
#
splash() {
	return 0
}
# This will override the splash() function...
if ! import_addon splash-functions.sh ; then
	[[ -f /sbin/splash-functions.sh ]] && source /sbin/splash-functions.sh
fi

# void profiling(...)
#
#  Notify bootsplash/whatever about important events.
#
profiling() {
	return 0
}
import_addon profiling-functions.sh

# void bootlog(...)
#
#  Notify bootlogger about important events.
bootlog() {
	return 0
}
[[ ${RC_BOOTLOG} == "yes" ]] && import_addon bootlogger.sh

# void get_bootconfig()
#
#    Get the BOOTLEVEL and SOFTLEVEL by setting
#    'bootlevel' and 'softlevel' via kernel
#    parameters.
#
get_bootconfig() {
	local copt= newbootlevel= newsoftlevel=

	if [[ -r /proc/cmdline ]] ; then
		for copt in $(</proc/cmdline) ; do
			case "${copt%=*}" in
				bootlevel)
					newbootlevel="${copt##*=}"
					;;
				softlevel)
					newsoftlevel="${copt##*=}"
					;;
			esac
		done
	fi

	if [[ -n ${newbootlevel} ]] ; then
		export BOOTLEVEL="${newbootlevel}"
	else
		export BOOTLEVEL="boot"
	fi

	if [[ -n ${newsoftlevel} ]] ; then
		export DEFAULTLEVEL="${newsoftlevel}"
	else
		export DEFAULTLEVEL="default"
	fi

	return 0
}

setup_defaultlevels() {
	get_bootconfig

	if get_bootparam "noconfigprofile" ; then
		export RC_USE_CONFIG_PROFILE="no"

	elif get_bootparam "configprofile" ; then
		export RC_USE_CONFIG_PROFILE="yes"
	fi

	if [[ ${RC_USE_CONFIG_PROFILE} == "yes" && -n ${DEFAULTLEVEL} ]] && \
	   [[ -d "/etc/runlevels/${BOOTLEVEL}.${DEFAULTLEVEL}" || \
	      -L "/etc/runlevels/${BOOTLEVEL}.${DEFAULTLEVEL}" ]] ; then
		export BOOTLEVEL="${BOOTLEVEL}.${DEFAULTLEVEL}"
	fi

	if [[ -z ${SOFTLEVEL} ]] ; then
		if [[ -f "${svcdir}/softlevel" ]] ; then
			export SOFTLEVEL=$(< "${svcdir}/softlevel")
		else
			export SOFTLEVEL="${BOOTLEVEL}"
		fi
	fi

	return 0
}

# void get_libdir(void)
#
#    prints the current libdir {lib,lib32,lib64}
#
get_libdir() {
	if [[ -n ${CONF_LIBDIR_OVERRIDE} ]] ; then
		CONF_LIBDIR="${CONF_LIBDIR_OVERRIDE}"
	elif [[ -x /usr/bin/portageq ]] ; then
		CONF_LIBDIR=$(/usr/bin/portageq envvar CONF_LIBDIR)
	fi
	echo "${CONF_LIBDIR:=lib}"
}

# char *ebuffer(char *buffer)
#
#    buffer e* commands to a file for output later
#    This is used for parallel startup/shutdown prettyness
#
ebuffer() {
	if  [[ $# == "0" ]] ; then
		echo "$_ebuffer"
	else
		_ebuffer="$1"
	fi

	return 0
}

# void esyslog(char* priority, char* tag, char* message)
#
#    use the system logger to log a message
#
esyslog() {
	local pri= tag=

	if [[ -x /usr/bin/logger ]] ; then
		pri="$1"
		tag="$2"

		shift 2
		[[ -z "$*" ]] && return 0

		/usr/bin/logger -p "${pri}" -t "${tag}" -- "$*"
	fi

	return 0
}

# void eindent(int num)
#
#    increase the indent used for e-commands.
#
eindent() {
	if [[ -n ${_ebuffer} ]] ; then
		echo -E "eindent $*" >> "${_ebuffer}"
		return 0
	fi

	local i="$1"
	(( i > 0 )) || (( i = RC_DEFAULT_INDENT ))
	esetdent $(( ${#RC_INDENTATION} + i ))
}

# void eoutdent(int num)
#
#    decrease the indent used for e-commands.
#
eoutdent() {
	if [[ -n ${_ebuffer} ]] ; then
		echo -E "eoutdent $*" >> "${_ebuffer}"
		return 0
	fi

	local i="$1"
	(( i > 0 )) || (( i = RC_DEFAULT_INDENT ))
	esetdent $(( ${#RC_INDENTATION} - i ))
}

# void esetdent(int num)
#
#    hard set the indent used for e-commands.
#    num defaults to 0
#
esetdent() {
	local i="$1"
	(( i < 0 )) && (( i = 0 ))
	RC_INDENTATION=$(printf "%${i}s" '')
}

# void einfo(char* message)
#
#    show an informative message (with a newline)
#
_einfo() {
	einfon "$*\n"
	LAST_E_CMD="einfo"
	return 0
}


# void einfon(char* message)
#
#    show an informative message (without a newline)
#
einfon() {
	[[ ${RC_QUIET_STDOUT} == "yes" ]] && return 0
	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
	echo -ne " ${GOOD}*${NORMAL} ${RC_INDENTATION}$*"
	LAST_E_CMD="einfon"
	return 0
}

# void ewarn(char* message)
#
#    show a warning message + log it
#
_ewarn() {
	if [[ ${RC_QUIET_STDOUT} == "yes" ]] ; then
		echo " $*"
	else
		[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
		echo -e " ${WARN}*${NORMAL} ${RC_INDENTATION}$*"
	fi

	local name="rc-scripts"
	[[ $0 != "/sbin/runscript.sh" ]] && name="${0##*/}"
	# Log warnings to system log
	esyslog "daemon.warning" "${name}" "$*"

	LAST_E_CMD="ewarn"
	return 0
}

# void eerror(char* message)
#
#    show an error message + log it
#
_eerror() {
	if [[ ${RC_QUIET_STDOUT} == "yes" ]] ; then
		echo " $*" >/dev/stderr
	else
		[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
		echo -e " ${BAD}*${NORMAL} ${RC_INDENTATION}$*"
	fi

	local name="rc-scripts"
	[[ $0 != "/sbin/runscript.sh" ]] && name="${0##*/}"
	# Log errors to system log
	esyslog "daemon.err" "${name}" "$*"

	LAST_E_CMD="eerror"
	return 1
}

# void ebegin(char* message)
#
#    show a message indicating the start of a process
#
ebegin() {
	[[ ${RC_QUIET_STDOUT} == "yes" ]] && return 0
	if [[ -n ${_ebuffer} ]] ; then
		echo -E "ebegin $*" >> "${_ebuffer}"
		return 0
	fi

	local msg="$*" dots spaces="${RC_DOT_PATTERN//?/ }"

	if [[ -n ${RC_DOT_PATTERN} ]] ; then
		dots=$(printf "%$((COLS - 3 - ${#RC_INDENTATION} - ${#msg} - 7))s" '')
		dots="${dots//${spaces}/${RC_DOT_PATTERN}}"
		msg="${msg}${dots}"
	else
		msg="${msg} ..."
	fi
	einfon "${msg}"
	[[ ${RC_ENDCOL} == "yes" ]] && echo

	LAST_E_LEN=$(( 3 + ${#RC_INDENTATION} + ${#msg} ))
	LAST_E_CMD="ebegin"
	return 0
}

# void _ewrap(char *cmd, char *...)
#
#   Enures that each einfo, ewarn and error statement does not exceed console
#   width - if it does then
_ewrap() {
	local cmd="$1"
	shift

	local c="${COLS}"
	[[ ${RC_ENDCOL} == "yes" ]] && c="${ENDCOL}"
	local p="$* "
	local max=$((COLS - ${#RC_INDENTATION} - 11))

	# Only use what we have space for
	local d="${p%%\\n*}"
	if [[ ${d} == "${p}" ]] ; then
		d="${p:0:${max}}"
	else
		d="${d} "
	fi

	# Grab the last whole word
	[[ ${#d} == "${max}" ]] && d="${d% *}"
	
	# Print that
	_${cmd} "${d}"

	# Recurse on the rest
	local r="$*"
	d="${d} "
	r="${r:${#d}}"
	[[ -n ${r} ]] && ${cmd} "${r}"
}

# wrappers for einfo, ewarn and eerror
# They work in exactly the same way, but allow us to use ebuffer and ewrap
einfo() {
	if [[ -n ${_ebuffer} && ${RC_QUIET_STDOUT} != "yes" ]] ; then
		echo "einfo ${*//\\/\\\\}" >> "${_ebuffer}"
		return 0
	fi

	_ewrap einfo "$*"
}
ewarn() {
	if [[ -n ${_ebuffer} ]] ; then
		echo "ewarn ${*//\\/\\\\}" >> "${_ebuffer}"
		return 0
	fi

	_ewrap ewarn "$*"
}
eerror() {
	if [[ -n ${_ebuffer} ]] ; then
		echo "eerror ${*//\\/\\\\}" >> "${_ebuffer}"
		return 1 
	fi

	_ewrap eerror "$*"
}


# void _eend(int error, char *efunc, char* errstr)
#
#    indicate the completion of process, called from eend/ewend
#    if error, show errstr via efunc
#
#    This function is private to functions.sh.  Do not call it from a
#    script.
#
_eend() {	
	local retval="${1:-0}" efunc="${2:-eerror}" msg=
	shift 2

	if [[ ${retval} == "0" ]] ; then
		[[ ${RC_QUIET_STDOUT} == "yes" ]] && return 0
		msg="${BRACKET}[ ${GOOD}ok${BRACKET} ]${NORMAL}"
	else
		[[ -n $* ]] && ${efunc} "$*"
		msg="${BRACKET}[ ${BAD}!!${BRACKET} ]${NORMAL}"
	fi

	if [[ ${RC_ENDCOL} == "yes" ]] ; then
		echo -e "${ENDCOL}  ${msg}"
	else
		[[ ${LAST_E_CMD} == ebegin ]] || LAST_E_LEN=0
		printf "%$(( COLS - LAST_E_LEN - 6 ))s%b\n" '' "${msg}"
	fi

	return ${retval}
}

# void eend(int error, char* errstr)
#
#    indicate the completion of process
#    if error, show errstr via eerror
#
eend() {
	local retval="${1:-0}"

	if [[ -n ${_ebuffer} ]] ; then
		[[ ${RC_QUIET_STDOUT} == "yes" && ${retval} == "0" ]] && return 0
		echo -E "eend $*" >> "${_ebuffer}"
		return "${retval}"
	fi

	shift
	_eend "${retval}" eerror "$*"
	LAST_E_CMD="eend"
	return ${retval}
}

# void ewend(int error, char* errstr)
#
#    indicate the completion of process
#    if error, show errstr via ewarn
#
ewend() {
	local retval="${1:-0}"

	if [[ -n ${_ebuffer} ]] ; then
		[[ ${RC_QUIET_STDOUT} == "yes" && ${retval} == "0" ]] && return 0
		echo -E "eend $*" >> "${_ebuffer}"
		return "${retval}"
	fi

	shift
	_eend "${retval}" ewarn "$*"
	LAST_E_CMD="ewend"
	return ${retval}
}

# void _eflush(void)
#
#   This the the backgroud job to flush the ebuffer
#
_eflush() {
	[[ ! -e ${_ebuffer} ]] && return

	local buf="${_ebuffer}"
	_ebuffer=

	# Change to $svcdir so we lock rc until we complete if rc is umounting it
	cd "${svcdir}"
	
	# Store our job buffer
	mv "${buf}" "${buf}.$$"

	# Wait until we can acquire a lock
	# mkfifo is nice as is returns 0 or 1 if we made the file or not
	while ! mkfifo "${svcdir}/ebuffer/.lock" 2>/dev/null ; do
		sleep 1
	done

	# OK, we have the lock so process the buffer
	local l= x= cmd= args= n=
	while read l ; do
		cmd="${l%% *}"
		args="${l#* }"
		[[ ${args} == "${l}" ]] && args=
		# Sanity check
		for x in ebegin eend ewend einfo ewarn eerror eindent eoutdent ; do
			[[ ${x} != ${cmd} ]] && continue
			case "${x}" in
				eend|ewend|eindent|eoutdent)
					n="${args%% *}"
					args="${args#* }"
					[[ ${args} == "${n}" ]] && args=
					${x} "${n}" "${args}"
					;;
				*)
					${x} "${args}"
					;;
			esac
			break
		done
	done < "${buf}.$$"

	rm -f "${buf}.$$" "${svcdir}/ebuffer/.lock"
	_ebuffer="${buf}"
}

# void _eflush(void)
#
#   Flush the ebuffer
#   The real work is done in a background job - this just launches it
#
eflush() {
	_eflush &
	return 0
}

# v-e-commands honor RC_VERBOSE which defaults to no.
# The condition is negated so the return value will be zero.
veinfo() { [[ ${RC_VERBOSE} != "yes" ]] || einfo "$@"; }
veinfon() { [[ ${RC_VERBOSE} != "yes" ]] || einfon "$@"; }
vewarn() { [[ ${RC_VERBOSE} != "yes" ]] || ewarn "$@"; }
veerror() { [[ ${RC_VERBOSE} != "yes" ]] || eerror "$@"; }
vebegin() { [[ ${RC_VERBOSE} != "yes" ]] || ebegin "$@"; }
veindent() { [[ ${RC_VERBOSE} != "yes" ]] || eindent "$@"; }
veoutdent() { [[ ${RC_VERBOSE} != "yes" ]] || eoutdent "$@"; }
veend() {
	[[ ${RC_VERBOSE} == "yes" ]] && { eend "$@"; return $?; }
	return ${1:-0}
}
vewend() {
	[[ ${RC_VERBOSE} == "yes" ]] && { ewend "$@"; return $?; }
	return ${1:-0}
}

# Many init scripts use uname - lets cache it
uname() {
	if [[ $# == "0" ]] ; then
		[[ -z ${_uname} ]] && declare -r _uname=$(/bin/uname)
		echo "${_uname}"
	else
		/bin/uname "$@"
	fi
}

# char *KV_major(string)
#
#    Return the Major (X of X.Y.Z) kernel version
#
KV_major() {
	[[ -z $1 ]] && return 1

	local KV="$@"
	echo "${KV%%.*}"
}

# char *KV_minor(string)
#
#    Return the Minor (Y of X.Y.Z) kernel version
#
KV_minor() {
	[[ -z $1 ]] && return 1

	local KV="$@"
	KV="${KV#*.}"
	echo "${KV%%.*}"
}

# char *KV_micro(string)
#
#    Return the Micro (Z of X.Y.Z) kernel version.
#
KV_micro() {
	[[ -z $1 ]] && return 1

	local KV="$@"
	KV="${KV#*.*.}"
	echo "${KV%%[^[:digit:]]*}"
}

# int KV_to_int(string)
#
#    Convert a string type kernel version (2.4.0) to an int (132096)
#    for easy compairing or versions ...
#
KV_to_int() {
	[[ -z $1 ]] && return 1

	local KV_MAJOR=$(KV_major "$1")
	local KV_MINOR=$(KV_minor "$1")
	local KV_MICRO=$(KV_micro "$1")
	local KV_int=$(( KV_MAJOR * 65536 + KV_MINOR * 256 + KV_MICRO ))

	# We make version 2.2.0 the minimum version we will handle as
	# a sanity check ... if its less, we fail ...
	if [[ ${KV_int} -ge 131584 ]] ; then
		echo "${KV_int}"
		return 0
	fi

	return 1
}

# int get_KV()
#
#    Return the kernel version (major, minor and micro concated) as an integer.
#    Assumes X and Y of X.Y.Z are numbers.  Also assumes that some leading
#    portion of Z is a number.
#    e.g. 2.4.25, 2.6.10, 2.6.4-rc3, 2.2.40-poop, 2.0.15+foo
#
_RC_GET_KV_CACHE=""
get_KV() {
	[[ -z ${_RC_GET_KV_CACHE} ]] \
		&& _RC_GET_KV_CACHE=$(uname -r)

	echo "$(KV_to_int "${_RC_GET_KV_CACHE}")"

	return $?
}

# bool get_bootparam(param)
#
#   return 0 if gentoo=param was passed to the kernel
#
#   EXAMPLE:  if get_bootparam "nodevfs" ; then ....
#
get_bootparam() {
	local x= copt= params= retval=1

	[[ ! -r /proc/cmdline ]] && return 1

	for copt in $(< /proc/cmdline) ; do
		if [[ ${copt%=*} == "gentoo" ]] ; then
			params=$(awk -v PARAMS="${copt##*=}" '
				BEGIN {
					split(PARAMS, nodes, ",")
					for (x in nodes)
						print nodes[x]
				}')

			# Parse gentoo option
			for x in ${params} ; do
				if [[ ${x} == "$1" ]] ; then
					retval=0
				fi
			done
		fi
	done

	return ${retval}
}

# Safer way to list the contents of a directory,
# as it do not have the "empty dir bug".
#
# char *dolisting(param)
#
#    print a list of the directory contents
#
#    NOTE: quote the params if they contain globs.
#          also, error checking is not that extensive ...
#
dolisting() {
	local x= y= mylist= mypath="$*"

	# Here we use bash file globbing instead of ls to save on forking
	for x in ${mypath} ; do
		[[ ! -e ${x} ]] && continue

		if [[ -L ${x} || -f ${x} ]] ; then
			mylist="${mylist} "${x}
		elif [[ -d ${x} ]] ; then
			[[ ${x%/} != "${x}" ]] && x=${x%/}
			
			for y in "${x}"/* ; do
				[[ -e ${y} ]] && mylist="${mylist} ${y}"
			done
		fi
	done

	echo "${mylist}"
}

# char *add_suffix(char * configfile)
#
#    Returns a config file name with the softlevel suffix
#    appended to it.  For use with multi-config services.
add_suffix() {
	if [[ ${RC_USE_CONFIG_PROFILE} != "yes" ]] ; then
		echo "$1"
		return 0
	fi

	local suffix="${SOFTLEVEL}"
	[[ ${SOFTLEVEL} == "${BOOTLEVEL}" \
	|| ${SOFTLEVEL} == "reboot" \
	|| ${SOFTLEVEL} == "shutdown" \
	|| ${SOFTLEVEL} == "single" ]] \
		&& suffix="${DEFAULTLEVEL}"
	if [[ -e "$1.${suffix}" ]] ; then
		echo "$1.${suffix}"
	else
		echo "$1"
	fi

	return 0
}

# char *get_base_ver()
#
#    get the version of baselayout that this system is running
#
get_base_ver() {
	[[ ! -r /etc/gentoo-release ]] && return 0
	local ver=$(</etc/gentoo-release)
	echo "${ver##* }"
}

# char get_mounts(void)
#
# The mount command, whilst having unportable output always exists
# We try to make it portable here, making out scripts easier to work with
# Ouput is mount point, node, fs. Provided that we never have a space in a
# node we will never break.
get_mounts() {
	local sedfoo='s/^\([^ ]*\) on \(.*\) (\([^,)]*\).*)$/\2 \1 \3/g'
	if [[ $(uname) == "Linux" ]] ;then
		sedfoo='s/^\([^ ]*\) on \(.*\) type \([^ ]*\) (.*)$/\2 \1 \3/g'
	fi
	LC_ALL=C mount | sed -e "${sedfoo}"
}

# Network filesystems list for common use in rc-scripts.
# This variable is used in is_net_fs and other places such as
# localmount.
NET_FS_LIST="afs cifs coda davfs fuse gfs ncpfs nfs nfs4 ocfs2 shfs smbfs"

# bool is_net_fs(path)
#
#   return 0 if path is the mountpoint of a networked filesystem
#
#   EXAMPLE:  if is_net_fs / ; then ...
#
is_net_fs() {
	local fstype
	# /proc/mounts is always accurate but may not always be available
	if [[ -e /proc/mounts ]] ; then
		fstype=$(sed -n -e '/^rootfs/!s:.* '"$1"' \([^ ]*\).*:\1:p' /proc/mounts)
	else
		fstype=$(mount | sed -n -e 's:.* on '"$1"' type \([^ ]*\).*:\1:p')
	fi
	[[ " ${NET_FS_LIST} " == *" ${fstype} "* ]]
	return $?
}

# bool is_net_fs(path)
#
#   return 0 if path is under unionfs control 
#
#   EXAMPLE:  if is_union_fs / ; then ...
#
is_union_fs() {
	[[ ! -x /sbin/unionctl ]] && return 1
	unionctl "$1" --list &>/dev/null
}

# bool is_uml_sys()
#
#   return 0 if the currently running system is User Mode Linux
is_uml_sys() {
    [[ -e /proc/cpuinfo ]] || return 1
    [[ $(</proc/cpuinfo) == *"UML"* ]]
}

# bool is_vps_sys()
#
#   return 0 if the currently running system is a Linux VServer or OpenVZ
is_vps_sys() {
    [[ -e /proc/self/status ]] || return 1
    [[ $'\n'$(</proc/self/status) =~ $'\n''s_context:[[:space:]]*[1-9]' \
	|| $'\n'$(</proc/self/status) =~ $'\n''VxID:[[:space:]]*[1-9]' \
    || $'\n'$(</proc/self/status) =~ $'\n''envID:[[:space:]]*[1-9]' ]]
}

# bool is_xenU_sys()
#
#   return 0 if the currently running system is an unprivileged Xen domain
is_xenU_sys() {
	[[ ! -d /proc/xen ]] && return 1
	[[ ! -r /proc/xen/capabilities ]] && return 1
	[[ ! $(</proc/xen/capabilities) == *"control_d"* ]]
}

# bool get_mount_fstab(path)
#
#   return the parameters to pass to the mount command generated from fstab
#
#   EXAMPLE: cmd=$( get_mount_fstab /proc )
#            cmd=${cmd:--t proc none /proc}
#            mount -n ${cmd}
#
get_mount_fstab() {
	awk '$1 ~ "^#" { next }
	     $2 == "'$*'" { stab="-t "$3" -o "$4" "$1" "$2; }
	     END { print stab; }
	' /etc/fstab
}

# char *reverse_list(list)
#
#   Returns the reversed order of list
#
reverse_list() {
	for (( i = $# ; i > 0 ; --i )) ; do
		echo -n "${!i} "
	done
}

# void start_addon(addon)
#
#   Starts addon.
#
start_addon() {
	local addon="$1"
	(import_addon "${addon}-start.sh")
	return 0
}

# void stop_addon(addon)
#
#   Stops addon.
#
stop_addon() {
	local addon=$1
	(import_addon "${addon}-stop.sh")
	return 0
}

# bool is_older_than(reference, files/dirs to check)
#
#   return 0 if any of the files/dirs are newer than
#   the reference file
#
#   EXAMPLE: if is_older_than a.out *.o ; then ...
is_older_than() {
	local x= ref="$1"
	shift

	for x in "$@" ; do
		if [[ -d ${x} ]] ; then
			is_older_than "${ref}" "${x}"/* && return 0
		elif [[ ${x} -nt ${ref} ]] ; then
			return 0
		fi
	done

	return 1
}

# char* bash_variable(char *variable)
#
#   Turns the given variable into something that bash can use
#   Basically replaces anything not a-z,A-Z into a _
#
bash_variable() {
	local args="$@"
	LC_ALL=C echo "${args//[![:word:]]/_}"
}

# void requote()
#
#   Requotes params so they're suitable to be eval'd, just like this would:
#   set -- 1 2 "3 4"
#   /usr/bin/getopt -- '' "$@" | sed 's/^ -- //'
#
requote() {
	local q=\'
	set -- "${@//\'/$q\'$q}"	# quote inner instances of '
	set -- "${@/#/$q}"				# add ' to start of each param
	set -- "${@/%/$q}"				# add ' to end of each param
	echo "$*"
}

# char* uniqify(char *arg, ...)
#
#   Ensure that params are unique
#
uniqify() {
    local result= x=
    while [[ -n "$1" ]] ; do
		[[ " ${result} " != *" $1 "* ]] && result="${result} $1"
		shift
	done
    echo "${result# *}"
}

##############################################################################
#                                                                            #
# This should be the last code in here, please add all functions above!!     #
#                                                                            #
# *** START LAST CODE ***                                                    #
#                                                                            #
##############################################################################

if [[ -z ${EBUILD} ]] ; then
	# Setup a basic $PATH.  Just add system default to existing.
	# This should solve both /sbin and /usr/sbin not present when
	# doing 'su -c foo', or for something like:  PATH= rcscript start
	PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/sbin:${PATH}"

	if [[ -z ${CONSOLETYPE} ]] ; then
		export CONSOLETYPE=$(/sbin/consoletype 2>/dev/null)
	fi
	if [[ ${CONSOLETYPE} == "serial" ]] ; then
		RC_NOCOLOR="yes"
		RC_ENDCOL="no"
	fi

	for arg in "$@" ; do
		case "${arg}" in
			# Lastly check if the user disabled it with --nocolor argument
			--nocolor|-nc)
				RC_NOCOLOR="yes"
				;;
		esac
	done

	setup_defaultlevels

	# If we are not /sbin/rc then ensure that we cannot change level variables
	if [[ -n ${BASH_SOURCE} \
		&& ${BASH_SOURCE[${#BASH_SOURCE[@]}-1]} != "/sbin/rc" ]] ; then
		declare -r BOOTLEVEL DEFAULTLEVEL SOFTLEVEL
	fi
else
	# Should we use colors ?
	if [[ $* != *depend* ]] ; then
		# Check user pref in portage
		RC_NOCOLOR=$(portageq envvar NOCOLOR 2>/dev/null)
		[[ ${RC_NOCOLOR} == "true" ]] && RC_NOCOLOR="yes"
	else
		# We do not want colors during emerge depend
		RC_NOCOLOR="yes"
		# No output is seen during emerge depend, so this is not needed.
		RC_ENDCOL="no"
	fi
fi

if [[ -n ${EBUILD} && $* == *depend* ]] ; then
	# We do not want stty to run during emerge depend
	COLS=80
elif [[ -z ${COLS} || -n ${COLUMNS} ]] ; then
	# Setup COLS and ENDCOL so eend can line up the [ ok ]
	COLS="${COLUMNS:-0}"		# bash's internal COLUMNS variable
	(( COLS == 0 )) && COLS=$(set -- `stty size 2>/dev/null` ; echo "$2")
	(( COLS > 0 )) || (( COLS = 80 ))	# width of [ ok ] == 7
fi

if [[ ${RC_ENDCOL} == "yes" ]] ; then
	ENDCOL=$'\e[A\e['$(( COLS - 9 ))'C'
else
	ENDCOL=''
fi

# Setup the colors so our messages all look pretty
if [[ ${RC_NOCOLOR} == "yes" ]] ; then
	unset GOOD WARN BAD NORMAL HILITE BRACKET
else
	if [[ ${RC_COLORS} == "pink" ]] ; then
		GOOD=$'\e[35;01m'
	else
		GOOD=$'\e[32;01m'
	fi
	WARN=$'\e[33;01m'
	BAD=$'\e[31;01m'
	HILITE=$'\e[36;01m'
	BRACKET=$'\e[34;01m'
	NORMAL=$'\e[0m'
fi

##############################################################################
#                                                                            #
# *** END LAST CODE ***                                                      #
#                                                                            #
# This should be the last code in here, please add all functions above!!     #
#                                                                            #
##############################################################################

# vim: set ts=4 :
